@@ -1,6 +1,6 @@
 /* Eye Of Gnome - Image Store
  *
- * Copyright (C) 2006 The Free Software Foundation
+ * Copyright (C) 2006-2007 The Free Software Foundation
  *
  * Author: Claudio Saavedra <csaavedra@alumnos.utalca.cl>
  *
@@ -52,9 +52,6 @@ struct _EogListStorePrivate {
 	GMutex *mutex;         /* Mutex for saving the jobs in the model */
 };
 
-gboolean
-eog_list_store_dump_contents (EogListStore *store);
-
 static void
 eog_list_store_finalize (GObject *object)
 {
@@ -228,6 +225,8 @@ is_file_in_list_store (EogListStore *store,
 		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
 				    EOG_LIST_STORE_EOG_IMAGE, &image,
 				    -1);
+		if (!image)
+			continue;
 		
 		uri = eog_image_get_uri (image);
 		str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
@@ -255,13 +254,12 @@ eog_job_thumbnail_cb (EogJobThumbnail *job, gpointer data)
 	GtkTreeIter iter;
 	gchar *filename;
 	EogImage *image;
+	GdkPixbuf *thumbnail;
 	
 	g_return_if_fail (EOG_IS_LIST_STORE (data));
 
 	store = EOG_LIST_STORE (data);
 
-/* 	thumbnail = g_object_ref (job->thumbnail); */
-	
 	filename = gnome_vfs_uri_to_string (job->uri_entry, GNOME_VFS_URI_HIDE_NONE);
 
 	if (is_file_in_list_store (store, filename, &iter)) {
@@ -270,25 +268,60 @@ eog_job_thumbnail_cb (EogJobThumbnail *job, gpointer data)
 				    -1);
 
 		eog_image_set_thumbnail (image, job->thumbnail);
+		/* getting the thumbnail, in case it needed transformations */
+		thumbnail = eog_image_get_thumbnail (image);
 
 		gtk_list_store_set (GTK_LIST_STORE (store), &iter, 
-				    EOG_LIST_STORE_THUMBNAIL, job->thumbnail,
+				    EOG_LIST_STORE_THUMBNAIL, thumbnail,
 				    EOG_LIST_STORE_THUMB_SET, TRUE,
 				    EOG_LIST_STORE_EOG_JOB, NULL,
 				    -1);
-/* 		g_object_unref (thumbnail); */
+		g_object_unref (thumbnail);
 	}
 
 	g_free (filename);
 }
 
+static void
+on_image_changed (EogImage *image, EogListStore *store)
+{
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	gint pos;
+	
+	pos = eog_list_store_get_pos_by_image (store, image);
+	path = gtk_tree_path_new_from_indices (pos, -1);
+
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
+	eog_list_store_thumbnail_refresh (store, &iter);
+	gtk_tree_path_free (path);
+}
+
+static void
+eog_list_store_remove (EogListStore *store, GtkTreeIter *iter)
+{
+	EogImage *image;
+
+	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
+			    EOG_LIST_STORE_EOG_IMAGE, &image,
+			    -1);
+
+	g_signal_handlers_disconnect_by_func (image, on_image_changed, store);
+	g_object_unref (image);
+
+	gtk_list_store_remove (GTK_LIST_STORE (store), iter);
+}
+
 void
 eog_list_store_append_image (EogListStore *store, EogImage *image)
 {
 	GtkTreeIter iter;
 
+	g_signal_connect (image, "changed",
+ 			  G_CALLBACK (on_image_changed),
+ 			  store);
+	
 	gtk_list_store_append (GTK_LIST_STORE (store), &iter);
-
 	gtk_list_store_set (GTK_LIST_STORE (store), &iter, 
 			    EOG_LIST_STORE_EOG_IMAGE, image, 
 			    EOG_LIST_STORE_THUMBNAIL, store->priv->busy_image,
@@ -320,17 +353,15 @@ vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
 	GtkTreeIter iter;
 	gchar *mimetype;
 
-	return;
-
 	switch (event_type) {
 	case GNOME_VFS_MONITOR_EVENT_CHANGED:
 		mimetype = gnome_vfs_get_mime_type (info_uri);
+
 		if (is_file_in_list_store (store, info_uri, &iter)) {
 			if (eog_image_is_supported_mime_type (mimetype)) {
-				eog_list_store_thumbnail_unset (store, &iter);
-				eog_list_store_thumbnail_set (store, &iter);
+				eog_list_store_thumbnail_refresh (store, &iter);
 			} else {
-				gtk_list_store_remove (GTK_LIST_STORE (store), &iter);
+				eog_list_store_remove (store, &iter);
 			}
 		} else {
 			if (eog_image_is_supported_mime_type (mimetype)) {
@@ -339,13 +370,13 @@ vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
 				gnome_vfs_uri_unref (uri);
 			}
 		}
+
 		g_free (mimetype);
 		break;
 
 	case GNOME_VFS_MONITOR_EVENT_DELETED:
-
 		if (is_file_in_list_store (store, info_uri, &iter)) {
-			gtk_list_store_remove (GTK_LIST_STORE (store), &iter);
+			eog_list_store_remove (store, &iter);
 		}
 		break;
 
@@ -358,6 +389,14 @@ vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
 		break;
 
 	case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
+		mimetype = gnome_vfs_get_mime_type (info_uri);
+		if (is_file_in_list_store (store, info_uri, &iter) &&
+		    eog_image_is_supported_mime_type (mimetype)) {
+			eog_list_store_thumbnail_refresh (store, &iter);
+		}
+		g_free (mimetype);
+		break;
+
 	case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
 	case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
 		break;
@@ -434,7 +473,6 @@ eog_list_store_append_directory (EogListStore *store,
 				       GNOME_VFS_DIRECTORY_VISIT_DEFAULT,
 				       directory_visit_cb,
 				       &ctx);
-	
 }
 
 static gboolean
@@ -446,6 +484,7 @@ get_uri_info (GnomeVFSURI *uri, GnomeVFSFileInfo *info)
 	g_return_val_if_fail (info != NULL, FALSE);
 	
 	gnome_vfs_file_info_clear (info);
+
 	result = gnome_vfs_get_file_info_uri (uri, info,
 					      GNOME_VFS_FILE_INFO_DEFAULT |
 					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
@@ -490,14 +529,17 @@ eog_list_store_add_uris (EogListStore *store, GList *uri_list)
 				continue;
 
 			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
+				gchar *uri_str;
+
 				eog_list_store_append_directory (store, uri, info);
 
-				if (!is_file_in_list_store (store, 
-							    gnome_vfs_uri_to_string (initial_uri, 
-										     GNOME_VFS_URI_HIDE_NONE), 
+				uri_str = gnome_vfs_uri_to_string (initial_uri, 
+						       GNOME_VFS_URI_HIDE_NONE);
+				if (!is_file_in_list_store (store, uri_str, 
 							    &iter)) {
 					eog_list_store_append_image_from_uri (store, initial_uri);
 				}
+				g_free (uri_str);
 			} else {
 				eog_list_store_append_image_from_uri (store, initial_uri);
 			}
@@ -540,7 +582,7 @@ eog_list_store_remove_image (EogListStore *store, EogImage *image)
 	gnome_vfs_uri_unref (uri);
 	
 	if (is_file_in_list_store (store, file, &iter)) {
-		gtk_list_store_remove (GTK_LIST_STORE (store), &iter);
+		eog_list_store_remove (store, &iter);
 	}
 	g_free (file);
 }
@@ -633,30 +675,43 @@ eog_list_store_get_initial_pos (EogListStore *store)
 	return store->priv->initial_image;
 }
 
-void
-eog_list_store_thumbnail_set (EogListStore *store, 
-			      GtkTreeIter *iter)
+static void
+eog_list_store_remove_thumbnail_job (EogListStore *store, 
+				     GtkTreeIter *iter)
 {
 	EogJob *job;
-	EogImage *image;
-	gboolean *thumb_set;
-	GnomeVFSURI *uri;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
-			    EOG_LIST_STORE_THUMB_SET, &thumb_set,
+			    EOG_LIST_STORE_EOG_JOB, &job,
 			    -1);
 
-	if (thumb_set) {
-		return;
+	if (job != NULL) {
+		g_mutex_lock (store->priv->mutex);
+		eog_job_queue_remove_job (job);
+		gtk_list_store_set (GTK_LIST_STORE (store), iter,
+				    EOG_LIST_STORE_EOG_JOB, NULL,
+				    -1);
+		g_mutex_unlock (store->priv->mutex);
 	}
+	
+	
+}
+
+static void
+eog_list_store_add_thumbnail_job (EogListStore *store, GtkTreeIter *iter)
+{
+	EogImage *image;
+	EogJob *job;
+	GnomeVFSURI *uri;
+	
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
-			    EOG_LIST_STORE_EOG_IMAGE, &image, 
+			    EOG_LIST_STORE_EOG_IMAGE, &image,
 			    -1);
 	
 	uri = eog_image_get_uri (image);
 	job = eog_job_thumbnail_new (uri);
 	gnome_vfs_uri_unref (uri);
-
+	
 	g_signal_connect (job,
 			  "finished",
 			  G_CALLBACK (eog_job_thumbnail_cb),
@@ -672,32 +727,47 @@ eog_list_store_thumbnail_set (EogListStore *store,
 	g_object_unref (image);
 }
 
+void
+eog_list_store_thumbnail_set (EogListStore *store, 
+			      GtkTreeIter *iter)
+{
+	gboolean *thumb_set;
+
+	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
+			    EOG_LIST_STORE_THUMB_SET, &thumb_set,
+			    -1);
+
+	if (thumb_set) {
+		return;
+	}
+
+	eog_list_store_add_thumbnail_job (store, iter);
+}
+
 void
 eog_list_store_thumbnail_unset (EogListStore *store, 
 				GtkTreeIter *iter)
 {
 	EogImage *image;
-	EogJob *job;
 
+	eog_list_store_remove_thumbnail_job (store, iter);
+	
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
 			    EOG_LIST_STORE_EOG_IMAGE, &image,
-			    EOG_LIST_STORE_EOG_JOB, &job,
 			    -1);
-
-	if (job != NULL) {
-		g_mutex_lock (store->priv->mutex);
-		eog_job_queue_remove_job (job);
-		gtk_list_store_set (GTK_LIST_STORE (store), iter,
-				    EOG_LIST_STORE_EOG_JOB, NULL,
-				    -1);
-		g_mutex_unlock (store->priv->mutex);
-	}
-
 	eog_image_set_thumbnail (image, NULL);
 	g_object_unref (image);
 
 	gtk_list_store_set (GTK_LIST_STORE (store), iter,
 			    EOG_LIST_STORE_THUMBNAIL, store->priv->busy_image,
 			    EOG_LIST_STORE_THUMB_SET, FALSE,
-			    -1);	
+			    -1);
+}
+
+void
+eog_list_store_thumbnail_refresh (EogListStore *store,
+				  GtkTreeIter *iter)
+{
+	eog_list_store_remove_thumbnail_job (store, iter);
+	eog_list_store_add_thumbnail_job (store, iter);
 }
