@@ -25,6 +25,7 @@
 
 #include "eog-thumb-view.h"
 #include "eog-list-store.h"
+#include "eog-pixbuf-cell-renderer.h"
 #include "eog-image.h"
 
 #ifdef HAVE_EXIF
@@ -194,7 +195,7 @@ tb_on_visible_range_changed_cb (EogThumbView *tb,
 	
 	eog_thumb_view_update_visible_range (tb, gtk_tree_path_get_indices (path1) [0],
 					     gtk_tree_path_get_indices (path2) [0]);
-	
+
 	gtk_tree_path_free (path1);
 	gtk_tree_path_free (path2);
 }
@@ -368,18 +369,15 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 	ExifData *exif_data;
 #endif	
 
-	if (keyboard_mode) {
-		image = eog_thumb_view_get_first_selected_image (EOG_THUMB_VIEW (widget));
-	} else {
-		if (!gtk_icon_view_get_item_at_pos (GTK_ICON_VIEW (widget),
-						    x, y, &path, NULL)) {
-			return FALSE;
-		}
-
-		image = eog_thumb_view_get_image_from_path (EOG_THUMB_VIEW (widget), path);
-
-		gtk_tree_path_free (path);
+	if (!gtk_icon_view_get_tooltip_context (GTK_ICON_VIEW (widget), 
+						&x, &y, keyboard_mode,
+						NULL, &path, NULL)) {
+		return FALSE;
 	}
+	
+	image = eog_thumb_view_get_image_from_path (EOG_THUMB_VIEW (widget),
+						    path);
+	gtk_tree_path_free (path);
 
 	if (image == NULL) {
 		return FALSE;
@@ -422,7 +420,7 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 					  eog_image_get_caption (image),
 					  width, 
 					  height, 
-					  ngettext ("pixels", "pixels", height), 
+					  ngettext ("pixel", "pixels", height), 
 					  bytes,
 					  type_str);
 
@@ -442,6 +440,7 @@ tb_on_query_tooltip_cb (GtkWidget  *widget,
 		g_free (tooltip_string);
 
 		tooltip_string = tmp;
+		exif_data_unref (exif_data);
 	}
 #endif
 
@@ -462,15 +461,16 @@ eog_thumb_view_init (EogThumbView *tb)
 {
 	tb->priv = EOG_THUMB_VIEW_GET_PRIVATE (tb);
 	
-	tb->priv->pixbuf_cell = gtk_cell_renderer_pixbuf_new ();
+	tb->priv->pixbuf_cell = eog_pixbuf_cell_renderer_new ();
 
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (tb), 
 	      	  		    tb->priv->pixbuf_cell, 
 	      			    FALSE);
 	
 	g_object_set (tb->priv->pixbuf_cell, 
-	              "follow-state", TRUE, 
+	              "follow-state", FALSE, 
 	              "height", 100, 
+	              "width", 115, 
 	              "yalign", 0.5, 
 	              "xalign", 0.5, 
 	              NULL);
@@ -699,6 +699,8 @@ eog_thumb_view_select_single (EogThumbView *tb,
 		gtk_icon_view_unselect_all (GTK_ICON_VIEW (tb));
 		
 		switch (change) {
+		case EOG_THUMB_VIEW_SELECT_CURRENT:
+			break;
 		case EOG_THUMB_VIEW_SELECT_LEFT:
 			if (!gtk_tree_path_prev (path)) {
 				gtk_tree_path_free (path);
